@@ -54,7 +54,6 @@
 #  define RESOLVER_ENOMEM  ENOMEM
 #endif
 
-#include "system_win32.h"
 #include "urldata.h"
 #include "sendf.h"
 #include "hostip.h"
@@ -65,6 +64,15 @@
 #include "inet_ntop.h"
 #include "curl_threads.h"
 #include "connect.h"
+#include "strdup.h"
+
+#ifdef USE_ARES
+#include <ares.h>
+#ifdef USE_HTTPSRR
+#define USE_HTTPSRR_ARES 1 /* the combo */
+#endif
+#endif
+
 /* The last 3 #include files should be in this order */
 #include "curl_printf.h"
 #include "curl_memory.h"
@@ -145,48 +153,6 @@ static bool init_resolve_thread(struct Curl_easy *data,
                                 const char *hostname, int port,
                                 const struct addrinfo *hints);
 
-#ifdef _WIN32
-/* Thread sync data used by GetAddrInfoExW for win8+ */
-struct thread_sync_data_w8
-{
-  OVERLAPPED overlapped;
-  ADDRINFOEXW_ *res;
-  HANDLE cancel_ev;
-  ADDRINFOEXW_ hints;
-};
-#endif
-
-/* Data for synchronization between resolver thread and its parent */
-struct thread_sync_data {
-#ifdef _WIN32
-  struct thread_sync_data_w8 w8;
-#endif
-  curl_mutex_t *mtx;
-  int done;
-  int port;
-  char *hostname;        /* hostname to resolve, Curl_async.hostname
-                            duplicate */
-#ifndef CURL_DISABLE_SOCKETPAIR
-  struct Curl_easy *data;
-  curl_socket_t sock_pair[2]; /* socket pair */
-#endif
-  int sock_error;
-  struct Curl_addrinfo *res;
-#ifdef HAVE_GETADDRINFO
-  struct addrinfo hints;
-#endif
-  struct thread_data *td; /* for thread-self cleanup */
-};
-
-struct thread_data {
-#ifdef _WIN32
-  HANDLE complete_ev;
-#endif
-  curl_thread_t thread_hnd;
-  unsigned int poll_interval;
-  timediff_t interval_end;
-  struct thread_sync_data tsd;
-};
 
 static struct thread_sync_data *conn_thread_sync_data(struct Curl_easy *data)
 {
@@ -212,9 +178,11 @@ void destroy_thread_sync_data(struct thread_sync_data *tsd)
    * close one end of the socket pair (may be done in resolver thread);
    * the other end (for reading) is always closed in the parent thread.
    */
+#ifndef USE_EVENTFD
   if(tsd->sock_pair[1] != CURL_SOCKET_BAD) {
     wakeup_close(tsd->sock_pair[1]);
   }
+#endif
 #endif
   memset(tsd, 0, sizeof(*tsd));
 }
@@ -235,7 +203,7 @@ int init_thread_sync_data(struct thread_data *td,
   /* Treat the request as done until the thread actually starts so any early
    * cleanup gets done properly.
    */
-  tsd->done = 1;
+  tsd->done = TRUE;
 #ifdef HAVE_GETADDRINFO
   DEBUGASSERT(hints);
   tsd->hints = *hints;
@@ -251,7 +219,7 @@ int init_thread_sync_data(struct thread_data *td,
 
 #ifndef CURL_DISABLE_SOCKETPAIR
   /* create socket pair or pipe */
-  if(wakeup_create(&tsd->sock_pair[0]) < 0) {
+  if(wakeup_create(tsd->sock_pair, FALSE) < 0) {
     tsd->sock_pair[0] = CURL_SOCKET_BAD;
     tsd->sock_pair[1] = CURL_SOCKET_BAD;
     goto err_exit;
@@ -286,175 +254,41 @@ static CURLcode getaddrinfo_complete(struct Curl_easy *data)
 
   result = Curl_addrinfo_callback(data, tsd->sock_error, tsd->res);
   /* The tsd->res structure has been copied to async.dns and perhaps the DNS
-     cache.  Set our copy to NULL so destroy_thread_sync_data doesn't free it.
+     cache. Set our copy to NULL so destroy_thread_sync_data does not free it.
   */
   tsd->res = NULL;
 
   return result;
 }
 
-#ifdef _WIN32
-static VOID WINAPI
-query_complete(DWORD err, DWORD bytes, LPWSAOVERLAPPED overlapped)
-{
-  size_t ss_size;
-  const ADDRINFOEXW_ *ai;
-  struct Curl_addrinfo *ca;
-  struct Curl_addrinfo *cafirst = NULL;
-  struct Curl_addrinfo *calast = NULL;
-#ifdef __clang__
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wcast-align"
-#endif
-  struct thread_sync_data *tsd =
-    CONTAINING_RECORD(overlapped, struct thread_sync_data, w8.overlapped);
-#ifdef __clang__
-#pragma clang diagnostic pop
-#endif
-  struct thread_data *td = tsd->td;
-  const ADDRINFOEXW_ *res = tsd->w8.res;
-  int error = (int)err;
-  (void)bytes;
-
-  if(error == ERROR_SUCCESS) {
-    /* traverse the addrinfo list */
-
-    for(ai = res; ai != NULL; ai = ai->ai_next) {
-      size_t namelen = ai->ai_canonname ? wcslen(ai->ai_canonname) + 1 : 0;
-      /* ignore elements with unsupported address family, */
-      /* settle family-specific sockaddr structure size.  */
-      if(ai->ai_family == AF_INET)
-        ss_size = sizeof(struct sockaddr_in);
-#ifdef ENABLE_IPV6
-      else if(ai->ai_family == AF_INET6)
-        ss_size = sizeof(struct sockaddr_in6);
-#endif
-      else
-        continue;
-
-      /* ignore elements without required address info */
-      if(!ai->ai_addr || !(ai->ai_addrlen > 0))
-        continue;
-
-      /* ignore elements with bogus address size */
-      if((size_t)ai->ai_addrlen < ss_size)
-        continue;
-
-      ca = malloc(sizeof(struct Curl_addrinfo) + ss_size + namelen);
-      if(!ca) {
-        error = EAI_MEMORY;
-        break;
-      }
-
-      /* copy each structure member individually, member ordering, */
-      /* size, or padding might be different for each platform.    */
-      ca->ai_flags     = ai->ai_flags;
-      ca->ai_family    = ai->ai_family;
-      ca->ai_socktype  = ai->ai_socktype;
-      ca->ai_protocol  = ai->ai_protocol;
-      ca->ai_addrlen   = (curl_socklen_t)ss_size;
-      ca->ai_addr      = NULL;
-      ca->ai_canonname = NULL;
-      ca->ai_next      = NULL;
-
-      ca->ai_addr = (void *)((char *)ca + sizeof(struct Curl_addrinfo));
-      memcpy(ca->ai_addr, ai->ai_addr, ss_size);
-
-      if(namelen) {
-        size_t i;
-        ca->ai_canonname = (void *)((char *)ca->ai_addr + ss_size);
-        for(i = 0; i < namelen; ++i) /* convert wide string to ascii */
-          ca->ai_canonname[i] = (char)ai->ai_canonname[i];
-        ca->ai_canonname[namelen] = '\0';
-      }
-
-      /* if the return list is empty, this becomes the first element */
-      if(!cafirst)
-        cafirst = ca;
-
-      /* add this element last in the return list */
-      if(calast)
-        calast->ai_next = ca;
-      calast = ca;
-    }
-
-    /* if we failed, also destroy the Curl_addrinfo list */
-    if(error) {
-      Curl_freeaddrinfo(cafirst);
-      cafirst = NULL;
-    }
-    else if(!cafirst) {
-#ifdef EAI_NONAME
-      /* rfc3493 conformant */
-      error = EAI_NONAME;
-#else
-      /* rfc3493 obsoleted */
-      error = EAI_NODATA;
-#endif
-#ifdef USE_WINSOCK
-      SET_SOCKERRNO(error);
-#endif
-    }
-    tsd->res = cafirst;
-  }
-
-  if(tsd->w8.res) {
-    Curl_FreeAddrInfoExW(tsd->w8.res);
-    tsd->w8.res = NULL;
-  }
-
-  if(error) {
-    tsd->sock_error = SOCKERRNO?SOCKERRNO:error;
-    if(tsd->sock_error == 0)
-      tsd->sock_error = RESOLVER_ENOMEM;
-  }
-  else {
-    Curl_addrinfo_set_port(tsd->res, tsd->port);
-  }
-
-  Curl_mutex_acquire(tsd->mtx);
-  if(tsd->done) {
-    /* too late, gotta clean up the mess */
-    Curl_mutex_release(tsd->mtx);
-    destroy_thread_sync_data(tsd);
-    free(td);
-  }
-  else {
-#ifndef CURL_DISABLE_SOCKETPAIR
-    char buf[1];
-    if(tsd->sock_pair[1] != CURL_SOCKET_BAD) {
-      /* DNS has been resolved, signal client task */
-      buf[0] = 1;
-      if(swrite(tsd->sock_pair[1],  buf, sizeof(buf)) < 0) {
-        /* update sock_erro to errno */
-        tsd->sock_error = SOCKERRNO;
-      }
-    }
-#endif
-    tsd->done = 1;
-    Curl_mutex_release(tsd->mtx);
-    if(td->complete_ev)
-      SetEvent(td->complete_ev); /* Notify caller that the query completed */
-  }
-}
-#endif
 
 #ifdef HAVE_GETADDRINFO
 
 /*
  * getaddrinfo_thread() resolves a name and then exits.
  *
- * For builds without ARES, but with ENABLE_IPV6, create a resolver thread
+ * For builds without ARES, but with USE_IPV6, create a resolver thread
  * and wait on it.
  */
-static unsigned int CURL_STDCALL getaddrinfo_thread(void *arg)
+static
+#if defined(_WIN32_WCE) || defined(CURL_WINDOWS_UWP)
+DWORD
+#else
+unsigned int
+#endif
+CURL_STDCALL getaddrinfo_thread(void *arg)
 {
   struct thread_sync_data *tsd = (struct thread_sync_data *)arg;
   struct thread_data *td = tsd->td;
   char service[12];
   int rc;
 #ifndef CURL_DISABLE_SOCKETPAIR
+#ifdef USE_EVENTFD
+  const void *buf;
+  const uint64_t val = 1;
+#else
   char buf[1];
+#endif
 #endif
 
   msnprintf(service, sizeof(service), "%d", tsd->port);
@@ -462,7 +296,7 @@ static unsigned int CURL_STDCALL getaddrinfo_thread(void *arg)
   rc = Curl_getaddrinfo_ex(tsd->hostname, service, &tsd->hints, &tsd->res);
 
   if(rc) {
-    tsd->sock_error = SOCKERRNO?SOCKERRNO:rc;
+    tsd->sock_error = SOCKERRNO ? SOCKERRNO : rc;
     if(tsd->sock_error == 0)
       tsd->sock_error = RESOLVER_ENOMEM;
   }
@@ -480,15 +314,19 @@ static unsigned int CURL_STDCALL getaddrinfo_thread(void *arg)
   else {
 #ifndef CURL_DISABLE_SOCKETPAIR
     if(tsd->sock_pair[1] != CURL_SOCKET_BAD) {
-      /* DNS has been resolved, signal client task */
+#ifdef USE_EVENTFD
+      buf = &val;
+#else
       buf[0] = 1;
-      if(wakeup_write(tsd->sock_pair[1],  buf, sizeof(buf)) < 0) {
+#endif
+      /* DNS has been resolved, signal client task */
+      if(wakeup_write(tsd->sock_pair[1], buf, sizeof(buf)) < 0) {
         /* update sock_erro to errno */
         tsd->sock_error = SOCKERRNO;
       }
     }
 #endif
-    tsd->done = 1;
+    tsd->done = TRUE;
     Curl_mutex_release(tsd->mtx);
   }
 
@@ -500,7 +338,13 @@ static unsigned int CURL_STDCALL getaddrinfo_thread(void *arg)
 /*
  * gethostbyname_thread() resolves a name and then exits.
  */
-static unsigned int CURL_STDCALL gethostbyname_thread(void *arg)
+static
+#if defined(_WIN32_WCE) || defined(CURL_WINDOWS_UWP)
+DWORD
+#else
+unsigned int
+#endif
+CURL_STDCALL gethostbyname_thread(void *arg)
 {
   struct thread_sync_data *tsd = (struct thread_sync_data *)arg;
   struct thread_data *td = tsd->td;
@@ -521,7 +365,7 @@ static unsigned int CURL_STDCALL gethostbyname_thread(void *arg)
     free(td);
   }
   else {
-    tsd->done = 1;
+    tsd->done = TRUE;
     Curl_mutex_release(tsd->mtx);
   }
 
@@ -537,37 +381,28 @@ static void destroy_async_data(struct Curl_async *async)
 {
   if(async->tdata) {
     struct thread_data *td = async->tdata;
-    int done;
+    bool done;
 #ifndef CURL_DISABLE_SOCKETPAIR
     curl_socket_t sock_rd = td->tsd.sock_pair[0];
     struct Curl_easy *data = td->tsd.data;
 #endif
 
+#ifdef USE_HTTPSRR_ARES
+    ares_destroy(data->state.async.tdata->channel);
+#endif
     /*
      * if the thread is still blocking in the resolve syscall, detach it and
      * let the thread do the cleanup...
      */
     Curl_mutex_acquire(td->tsd.mtx);
     done = td->tsd.done;
-    td->tsd.done = 1;
+    td->tsd.done = TRUE;
     Curl_mutex_release(td->tsd.mtx);
 
     if(!done) {
-#ifdef _WIN32
-      if(td->complete_ev)
-        CloseHandle(td->complete_ev);
-      else
-#endif
       Curl_thread_destroy(td->thread_hnd);
     }
     else {
-#ifdef _WIN32
-      if(td->complete_ev) {
-        Curl_GetAddrInfoExCancel(&td->tsd.w8.cancel_ev);
-        WaitForSingleObject(td->complete_ev, INFINITE);
-        CloseHandle(td->complete_ev);
-      }
-#endif
       if(td->thread_hnd != curl_thread_t_null)
         Curl_thread_join(&td->thread_hnd);
 
@@ -590,6 +425,24 @@ static void destroy_async_data(struct Curl_async *async)
   async->hostname = NULL;
 }
 
+#ifdef USE_HTTPSRR_ARES
+static CURLcode resolve_httpsrr(struct Curl_easy *data,
+                                struct Curl_async *asp)
+{
+  int status = ares_init_options(&asp->tdata->channel, NULL, 0);
+  if(status != ARES_SUCCESS)
+    return CURLE_FAILED_INIT;
+
+  memset(&asp->tdata->hinfo, 0, sizeof(struct Curl_https_rrinfo));
+  ares_query_dnsrec(asp->tdata->channel,
+                    asp->hostname, ARES_CLASS_IN,
+                    ARES_REC_TYPE_HTTPS,
+                    Curl_dnsrec_done_cb, data, NULL);
+
+  return CURLE_OK;
+}
+#endif
+
 /*
  * init_resolve_thread() starts a new thread that performs the actual
  * resolve. This function returns before the resolve is done.
@@ -613,9 +466,6 @@ static bool init_resolve_thread(struct Curl_easy *data,
   asp->status = 0;
   asp->dns = NULL;
   td->thread_hnd = curl_thread_t_null;
-#ifdef _WIN32
-  td->complete_ev = NULL;
-#endif
 
   if(!init_thread_sync_data(td, hostname, port, hints)) {
     asp->tdata = NULL;
@@ -628,43 +478,8 @@ static bool init_resolve_thread(struct Curl_easy *data,
   if(!asp->hostname)
     goto err_exit;
 
-  /* The thread will set this to 1 when complete. */
-  td->tsd.done = 0;
-
-#ifdef _WIN32
-  if(Curl_isWindows8OrGreater && Curl_FreeAddrInfoExW &&
-     Curl_GetAddrInfoExCancel && Curl_GetAddrInfoExW) {
-#define MAX_NAME_LEN 256 /* max domain name is 253 chars */
-#define MAX_PORT_LEN 8
-    WCHAR namebuf[MAX_NAME_LEN];
-    WCHAR portbuf[MAX_PORT_LEN];
-    /* calculate required length */
-    int w_len = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, hostname,
-                                    -1, NULL, 0);
-    if((w_len > 0) && (w_len < MAX_NAME_LEN)) {
-      /* do utf8 conversion */
-      w_len = MultiByteToWideChar(CP_UTF8, 0, hostname, -1, namebuf, w_len);
-      if((w_len > 0) && (w_len < MAX_NAME_LEN)) {
-        swprintf(portbuf, MAX_PORT_LEN, L"%d", port);
-        td->tsd.w8.hints.ai_family = hints->ai_family;
-        td->tsd.w8.hints.ai_socktype = hints->ai_socktype;
-        td->complete_ev = CreateEvent(NULL, TRUE, FALSE, NULL);
-        if(!td->complete_ev) {
-          /* failed to start, mark it as done here for proper cleanup. */
-          td->tsd.done = 1;
-          goto err_exit;
-        }
-        err = Curl_GetAddrInfoExW(namebuf, portbuf, NS_DNS,
-                                  NULL, &td->tsd.w8.hints, &td->tsd.w8.res,
-                                  NULL, &td->tsd.w8.overlapped,
-                                  &query_complete, &td->tsd.w8.cancel_ev);
-        if(err != WSA_IO_PENDING)
-          query_complete(err, 0, &td->tsd.w8.overlapped);
-        return TRUE;
-      }
-    }
-  }
-#endif
+  /* The thread will set this TRUE when complete. */
+  td->tsd.done = FALSE;
 
 #ifdef HAVE_GETADDRINFO
   td->thread_hnd = Curl_thread_create(getaddrinfo_thread, &td->tsd);
@@ -672,13 +487,16 @@ static bool init_resolve_thread(struct Curl_easy *data,
   td->thread_hnd = Curl_thread_create(gethostbyname_thread, &td->tsd);
 #endif
 
-  if(!td->thread_hnd) {
+  if(td->thread_hnd == curl_thread_t_null) {
     /* The thread never started, so mark it as done here for proper cleanup. */
-    td->tsd.done = 1;
+    td->tsd.done = TRUE;
     err = errno;
     goto err_exit;
   }
-
+#ifdef USE_HTTPSRR_ARES
+  if(resolve_httpsrr(data, asp))
+    goto err_exit;
+#endif
   return TRUE;
 
 err_exit:
@@ -702,22 +520,9 @@ static CURLcode thread_wait_resolv(struct Curl_easy *data,
   DEBUGASSERT(data);
   td = data->state.async.tdata;
   DEBUGASSERT(td);
-#ifdef _WIN32
-  DEBUGASSERT(td->complete_ev || td->thread_hnd != curl_thread_t_null);
-#else
   DEBUGASSERT(td->thread_hnd != curl_thread_t_null);
-#endif
 
   /* wait for the thread to resolve the name */
-#ifdef _WIN32
-  if(td->complete_ev) {
-    WaitForSingleObject(td->complete_ev, INFINITE);
-    CloseHandle(td->complete_ev);
-    if(entry)
-      result = getaddrinfo_complete(data);
-  }
-  else
-#endif
   if(Curl_thread_join(&td->thread_hnd)) {
     if(entry)
       result = getaddrinfo_complete(data);
@@ -751,8 +556,8 @@ void Curl_resolver_kill(struct Curl_easy *data)
 {
   struct thread_data *td = data->state.async.tdata;
 
-  /* If we're still resolving, we must wait for the threads to fully clean up,
-     unfortunately.  Otherwise, we can simply cancel to clean up any resolver
+  /* If we are still resolving, we must wait for the threads to fully clean up,
+     unfortunately. Otherwise, we can simply cancel to clean up any resolver
      data. */
   if(td && td->thread_hnd != curl_thread_t_null
      && (data->set.quick_exit != 1L))
@@ -789,7 +594,7 @@ CURLcode Curl_resolver_is_resolved(struct Curl_easy *data,
                                    struct Curl_dns_entry **entry)
 {
   struct thread_data *td = data->state.async.tdata;
-  int done = 0;
+  bool done = FALSE;
 
   DEBUGASSERT(entry);
   *entry = NULL;
@@ -798,6 +603,10 @@ CURLcode Curl_resolver_is_resolved(struct Curl_easy *data,
     DEBUGASSERT(td);
     return CURLE_COULDNT_RESOLVE_HOST;
   }
+#ifdef USE_HTTPSRR_ARES
+  if(Curl_ares_perform(data->state.async.tdata->channel, 0) < 0)
+    return CURLE_UNRECOVERABLE_POLL;
+#endif
 
   Curl_mutex_acquire(td->tsd.mtx);
   done = td->tsd.done;
@@ -811,12 +620,23 @@ CURLcode Curl_resolver_is_resolved(struct Curl_easy *data,
       destroy_async_data(&data->state.async);
       return result;
     }
+#ifdef USE_HTTPSRR_ARES
+    {
+      struct Curl_https_rrinfo *lhrr =
+        Curl_memdup(&td->hinfo, sizeof(struct Curl_https_rrinfo));
+      if(!lhrr) {
+        destroy_async_data(&data->state.async);
+        return CURLE_OUT_OF_MEMORY;
+      }
+      data->state.async.dns->hinfo = lhrr;
+    }
+#endif
     destroy_async_data(&data->state.async);
     *entry = data->state.async.dns;
   }
   else {
     /* poll for name lookup done with exponential backoff up to 250ms */
-    /* should be fine even if this converts to 32 bit */
+    /* should be fine even if this converts to 32-bit */
     timediff_t elapsed = Curl_timediff(Curl_now(),
                                        data->progress.t_startsingle);
     if(elapsed < 0)
@@ -845,18 +665,28 @@ int Curl_resolver_getsock(struct Curl_easy *data, curl_socket_t *socks)
   timediff_t milli;
   timediff_t ms;
   struct resdata *reslv = (struct resdata *)data->state.async.resolver;
+  int socketi = 0;
 #ifndef CURL_DISABLE_SOCKETPAIR
   struct thread_data *td = data->state.async.tdata;
 #else
   (void)socks;
 #endif
 
+#ifdef USE_HTTPSRR_ARES
+  if(data->state.async.tdata) {
+    ret_val = Curl_ares_getsock(data, data->state.async.tdata->channel, socks);
+    for(socketi = 0; socketi < (MAX_SOCKSPEREASYHANDLE - 1); socketi++)
+      if(!ARES_GETSOCK_READABLE(ret_val, socketi) &&
+         !ARES_GETSOCK_WRITABLE(ret_val, socketi))
+        break;
+  }
+#endif
 #ifndef CURL_DISABLE_SOCKETPAIR
   if(td) {
     /* return read fd to client for polling the DNS resolution status */
-    socks[0] = td->tsd.sock_pair[0];
+    socks[socketi] = td->tsd.sock_pair[0];
     td->tsd.data = data;
-    ret_val = GETSOCK_READSOCK(0);
+    ret_val = GETSOCK_READSOCK(socketi);
   }
   else {
 #endif
@@ -932,7 +762,7 @@ struct Curl_addrinfo *Curl_resolver_getaddrinfo(struct Curl_easy *data,
 
   memset(&hints, 0, sizeof(hints));
   hints.ai_family = pf;
-  hints.ai_socktype = (data->conn->transport == TRNSPRT_TCP)?
+  hints.ai_socktype = (data->conn->transport == TRNSPRT_TCP) ?
     SOCK_STREAM : SOCK_DGRAM;
 
   reslv->start = Curl_now();
